#include "cdstd.h"
#include <UnitTest++.h>
#include "cddefines.h"
#include "thirdparty.h"
#include "service.h"

namespace {

	TEST(TestFFmtRead)
	{
		bool lgEol;
		char buf[128];
		init_genrand( 330327317L );
		for( int i=0; i < 4096; ++i )
		{
			double x = pow(10.,genrand_real1()*600. - 300.);
			if( genrand_int32()%1 )
				x *= -1.;
			sprintf( buf, " %.16e", x );
			long j = 1;
			double y = FFmtRead( buf, &j, 128, &lgEol );
			CHECK( fp_equal( x, y, 2 ) );
			CHECK( !lgEol );
		}
	}

	TEST(TestPowi)
	{
		init_genrand( 395327317L );
		for( int i=0; i < 2048; ++i )
		{
			double arg1 = genrand_real1()*200. - 100.;
			long arg2 = (genrand_int32()%200) - 100;
			CHECK( fp_equal( powi(arg1,arg2), pow(arg1,(double)arg2), max(abs(arg2),3) ) );
		}
	}

	TEST(TestPowpq)
	{
		init_genrand( 515337317L );
		for( int i=0; i < 1024; ++i )
		{
			double arg1 = genrand_real1()*1.e50;
			int arg2 = (genrand_int32()%9) + 1;
			int arg3 = (genrand_int32()%8) + 2;
			// powpq() may be more accurate than pow(), e.g. for powpq(x,1,3)...
			CHECK( fp_equal( powpq(arg1,arg2,arg3), pow(arg1,(double)arg2/(double)arg3), 128 ) );
		}
		for( int i=0; i < 1024; ++i )
		{
			double arg1 = genrand_real1()*1.e50;
			int arg2 = -((genrand_int32()%9) + 1);
			int arg3 = (genrand_int32()%8) + 2;
			CHECK( fp_equal( powpq(arg1,arg2,arg3), pow(arg1,(double)arg2/(double)arg3), 128 ) );
		}
	}
}
