/* This file is part of Cloudy and is copyright (C)1978-2015 by Gary J. Ferland and
 * others.  For conditions of distribution and use see copyright notice in license.txt */

#ifndef SERVICE_H_
#define SERVICE_H_

#include <string>
#include <vector>

/** split_mode defines how the routine Split generates substrings
 * SPM_RELAX: multiple adjacent separators will be coalesced into one
 *            this way you can never get an empty substring
 * SPM_KEEP_EMPTY: multiple adjacent separators will result in empty
 *                 substrings to be added to the list
 * SPM_STRICT: empty substrings are illegal */
typedef enum { SPM_RELAX, SPM_KEEP_EMPTY, SPM_STRICT } split_mode;

// remove whitespace from the end of a string
void trimTrailingWhiteSpace( std::string &str );
// remove whitespace form the end of a char array
void trimTrailingWhiteSpace( char *str );

/** Split: split a string into substrings using "sep" as separator */
void Split(const std::string& str,   // input string
	   const std::string& sep,   // separator, may be multiple characters
	   std::vector<std::string>& lst, // the separated items will be appended here
	   split_mode mode);    // see above

inline bool FindAndReplace(string& str,
			   const string& substr,
			   const string& newstr)
{
	string::size_type ptr = str.find( substr );
	if( ptr != string::npos )
		str.replace( ptr, substr.length(), newstr );
	return ptr != string::npos;
}

inline bool FindAndErase(string& str,
			 const string& substr)
{
	return FindAndReplace( str, substr, "" );
}

void service(double tau, double a, double beta);

#endif /* SERVICE_ */
